@@ -6,6 +6,7 @@
  * $Id:
  */
 
+#if APIVERSNUM < 10507
 #include "i18n.h"
 
  const tI18nPhrase Phrases[] = {
@@ -1354,4 +1355,4 @@
   },
   { NULL }
   };
-
+#endif
