@@ -3,7 +3,6 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id$
  */
 
 #ifndef __SKINSOPPALUSIKKA_CONFIG_H
@@ -12,20 +11,19 @@
 struct cSoppalusikkaConfig
 {
 private:
-  char logodir[255];
+  cString logodir;
 public:
   cSoppalusikkaConfig();
   ~cSoppalusikkaConfig();
   void SetLogoDir(const char *logodirP);
-  char *GetLogoDir(void) { return logodir; }
-  int usesinglearea;
+  const char *GetLogoDir(void) { return logodir; }
   int showauxinfo;
   int showlogo;
   int showvps;
+  int showduration; // 0 = remaining; 1 = total
   int showsymbols;
   int showprogressbar;
   int cachesize;
-  int usechannelid;
 };
 
 extern cSoppalusikkaConfig SoppalusikkaConfig;
