@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: channels.c 1.54 2007/07/21 14:55:01 kls Exp $
+ * $Id: channels.c 1.60 2008/03/05 16:42:50 kls Exp $
  */
 
 #include "channels.h"
@@ -18,7 +18,7 @@
 // format characters in order to allow any number of blanks after a numeric
 // value!
 
-// -- Channel Parameter Maps -------------------------------------------------
+// --- Channel Parameter Maps ------------------------------------------------
 
 const tChannelParameterMap InversionValues[] = {
   {   0, INVERSION_OFF },
@@ -123,7 +123,7 @@ int MapToDriver(int Value, const tChannelParameterMap *Map)
   return -1;
 }
 
-// -- tChannelID -------------------------------------------------------------
+// --- tChannelID ------------------------------------------------------------
 
 const tChannelID tChannelID::InvalidID;
 
@@ -158,7 +158,7 @@ tChannelID &tChannelID::ClrPolarization(void)
   return *this;
 }
 
-// -- cChannel ---------------------------------------------------------------
+// --- cChannel --------------------------------------------------------------
 
 cChannel::cChannel(void)
 {
@@ -166,6 +166,7 @@ cChannel::cChannel(void)
   shortName = strdup("");
   provider = strdup("");
   portalName = strdup("");
+  pluginParam = strdup("");
   memset(&__BeginData__, 0, (char *)&__EndData__ - (char *)&__BeginData__);
   inversion    = INVERSION_AUTO;
   bandwidth    = BANDWIDTH_AUTO;
@@ -187,6 +188,7 @@ cChannel::cChannel(const cChannel &Channel)
   shortName = NULL;
   provider = NULL;
   portalName = NULL;
+  pluginParam = NULL;
   schedule     = NULL;
   linkChannels = NULL;
   refChannel   = NULL;
@@ -215,6 +217,7 @@ cChannel::~cChannel()
   free(shortName);
   free(provider);
   free(portalName);
+  free(pluginParam);
 }
 
 cChannel& cChannel::operator= (const cChannel &Channel)
@@ -223,6 +226,7 @@ cChannel& cChannel::operator= (const cChannel &Channel)
   shortName = strcpyrealloc(shortName, Channel.shortName);
   provider = strcpyrealloc(provider, Channel.provider);
   portalName = strcpyrealloc(portalName, Channel.portalName);
+  pluginParam = strcpyrealloc(pluginParam, Channel.pluginParam);
   memcpy(&__BeginData__, &Channel.__BeginData__, (char *)&Channel.__EndData__ - (char *)&Channel.__BeginData__);
   return *this;
 }
@@ -280,9 +284,26 @@ void cChannel::CopyTransponderData(const cChannel *Channel)
      transmission = Channel->transmission;
      guard        = Channel->guard;
      hierarchy    = Channel->hierarchy;
+     if (IsPlug())  pluginParam = strcpyrealloc(pluginParam, Channel->pluginParam);
      }
 }
 
+bool cChannel::SetPlugTransponderData(int Source, int Frequency, const char *PluginParam)
+{
+  if (source != Source || frequency != Frequency || (strcmp(pluginParam, PluginParam) != 0)) {
+     if (Number()) {
+        dsyslog("changing transponder data of channel %d from %s:%d:%s to %s:%d:%s", Number(), *cSource::ToString(source), frequency, pluginParam, *cSource::ToString(Source), Frequency, PluginParam);
+        modification |= CHANNELMOD_TRANSP;
+        Channels.SetModified();
+        }
+     source = Source;
+     frequency = Frequency;
+     pluginParam = strcpyrealloc(pluginParam, PluginParam);
+     schedule = NULL;
+     }
+  return true;
+}
+
 bool cChannel::SetSatTransponderData(int Source, int Frequency, char Polarization, int Srate, int CoderateH)
 {
   // Workarounds for broadcaster stupidity:
@@ -407,6 +428,18 @@ void cChannel::SetPortalName(const char *PortalName)
      }
 }
 
+void cChannel::SetPluginParam(const char *PluginParam)
+{
+  if (!isempty(PluginParam) && strcmp(pluginParam, PluginParam) != 0) {
+     if (Number()) {
+        dsyslog("changing plugin parameters of channel %d from '%s' to '%s'", Number(), pluginParam, PluginParam);
+        modification |= CHANNELMOD_TRANSP;
+        Channels.SetModified();
+        }
+     pluginParam = strcpyrealloc(pluginParam, PluginParam);
+     }
+}
+
 #define STRDIFF 0x01
 #define VALDIFF 0x02
 
@@ -440,12 +473,12 @@ static int IntArrayToString(char *s, const int *a, int Base = 10, const char n[]
   return q - s;
 }
 
-void cChannel::SetPids(int Vpid, int Ppid, int *Apids, char ALangs[][MAXLANGCODE2], int *Dpids, char DLangs[][MAXLANGCODE2], int Tpid)
+void cChannel::SetPids(int Vpid, int Ppid, int *Apids, char ALangs[][MAXLANGCODE2], int *Dpids, char DLangs[][MAXLANGCODE2], int *Spids, char SLangs[][MAXLANGCODE2], int Tpid)
 {
   int mod = CHANNELMOD_NONE;
   if (vpid != Vpid || ppid != Ppid || tpid != Tpid)
      mod |= CHANNELMOD_PIDS;
-  int m = IntArraysDiffer(apids, Apids, alangs, ALangs) | IntArraysDiffer(dpids, Dpids, dlangs, DLangs);
+  int m = IntArraysDiffer(apids, Apids, alangs, ALangs) | IntArraysDiffer(dpids, Dpids, dlangs, DLangs) | IntArraysDiffer(spids, Spids, slangs, SLangs);
   if (m & STRDIFF)
      mod |= CHANNELMOD_LANGS;
   if (m & VALDIFF)
@@ -468,7 +501,16 @@ void cChannel::SetPids(int Vpid, int Ppid, int *Apids, char ALangs[][MAXLANGCODE
         q += IntArrayToString(q, Dpids, 10, DLangs);
         }
      *q = 0;
-     dsyslog("changing pids of channel %d from %d+%d:%s:%d to %d+%d:%s:%d", Number(), vpid, ppid, OldApidsBuf, tpid, Vpid, Ppid, NewApidsBuf, Tpid);
+     const int SBufferSize = MAXSPIDS * (5 + 1 + MAXLANGCODE2) + 10; // 5 digits plus delimiting ',' or ';' plus optional '=cod', +10: paranoia
+     char OldSpidsBuf[SBufferSize];
+     char NewSpidsBuf[SBufferSize];
+     q = OldSpidsBuf;
+     q += IntArrayToString(q, spids, 10, slangs);
+     *q = 0;
+     q = NewSpidsBuf;
+     q += IntArrayToString(q, Spids, 10, SLangs);
+     *q = 0;
+     dsyslog("changing pids of channel %d from %d+%d:%s:%s:%d to %d+%d:%s:%s:%d", Number(), vpid, ppid, OldApidsBuf, OldSpidsBuf, tpid, Vpid, Ppid, NewApidsBuf, NewSpidsBuf, Tpid);
      vpid = Vpid;
      ppid = Ppid;
      for (int i = 0; i < MAXAPIDS; i++) {
@@ -481,6 +523,11 @@ void cChannel::SetPids(int Vpid, int Ppid, int *Apids, char ALangs[][MAXLANGCODE
          strn0cpy(dlangs[i], DLangs[i], MAXLANGCODE2);
          }
      dpids[MAXDPIDS] = 0;
+     for (int i = 0; i < MAXSPIDS; i++) {
+         spids[i] = Spids[i];
+         strn0cpy(slangs[i], SLangs[i], MAXLANGCODE2);
+         }
+     spids[MAXSPIDS] = 0;
      tpid = Tpid;
      modification |= mod;
      Channels.SetModified();
@@ -579,7 +626,7 @@ cString cChannel::ParametersToString(void) const
   if (isdigit(type))
      type = 'S';
 #define ST(s) if (strchr(s, type))
-  char buffer[64];
+  char buffer[256];
   char *q = buffer;
   *q = 0;
   ST(" S ")  q += sprintf(q, "%c", polarization);
@@ -591,6 +638,7 @@ cString cChannel::ParametersToString(void) const
   ST("  T")  q += PrintParameter(q, 'T', MapToUser(transmission, TransmissionValues));
   ST("  T")  q += PrintParameter(q, 'G', MapToUser(guard, GuardValues));
   ST("  T")  q += PrintParameter(q, 'Y', MapToUser(hierarchy, HierarchyValues));
+  ST("P  ")  snprintf(buffer, sizeof(buffer), "%s", pluginParam);
   return buffer;
 }
 
@@ -612,7 +660,7 @@ static const char *ParseParameter(const char *s, int &Value, const tChannelParam
 
 bool cChannel::StringToParameters(const char *s)
 {
-  while (s && *s) {
+  while (s && *s && !IsPlug()) {
         switch (toupper(*s)) {
           case 'B': s = ParseParameter(s, bandwidth, BandwidthValues); break;
           case 'C': s = ParseParameter(s, coderateH, CoderateValues); break;
@@ -644,12 +692,12 @@ cString cChannel::ToText(const cChannel *Channel)
      q += sprintf(q, ";%s", Channel->provider);
   *q = 0;
   strreplace(FullName, ':', '|');
-  char *buffer;
+  cString buffer;
   if (Channel->groupSep) {
      if (Channel->number)
-        asprintf(&buffer, ":@%d %s\n", Channel->number, FullName);
+        buffer = cString::sprintf(":@%d %s\n", Channel->number, FullName);
      else
-        asprintf(&buffer, ":%s\n", FullName);
+        buffer = cString::sprintf(":%s\n", FullName);
      }
   else {
      char vpidbuf[32];
@@ -671,9 +719,9 @@ cString cChannel::ToText(const cChannel *Channel)
      q = caidbuf;
      q += IntArrayToString(q, Channel->caids, 16);
      *q = 0;
-     asprintf(&buffer, "%s:%d:%s:%s:%d:%s:%s:%d:%s:%d:%d:%d:%d\n", FullName, Channel->frequency, *Channel->ParametersToString(), *cSource::ToString(Channel->source), Channel->srate, vpidbuf, apidbuf, Channel->tpid, caidbuf, Channel->sid, Channel->nid, Channel->tid, Channel->rid);
+     buffer = cString::sprintf("%s:%d:%s:%s:%d:%s:%s:%d:%s:%d:%d:%d:%d\n", FullName, Channel->frequency, *Channel->ParametersToString(), *cSource::ToString(Channel->source), Channel->srate, vpidbuf, apidbuf, Channel->tpid, caidbuf, Channel->sid, Channel->nid, Channel->tid, Channel->rid);
      }
-  return cString(buffer, true);
+  return buffer;
 }
 
 cString cChannel::ToText(void) const
@@ -722,7 +770,7 @@ bool cChannel::Parse(const char *s)
         dpids[0] = 0;
         ok = false;
         if (parambuf && sourcebuf && vpidbuf && apidbuf) {
-           ok = StringToParameters(parambuf) && (source = cSource::FromString(sourcebuf)) >= 0;
+           ok = ((source = cSource::FromString(sourcebuf)) >= 0) && StringToParameters(parambuf);
 
            char *p = strchr(vpidbuf, '+');
            if (p)
@@ -813,6 +861,7 @@ bool cChannel::Parse(const char *s)
            shortName = strcpyrealloc(shortName, p);
            }
         name = strcpyrealloc(name, namebuf);
+        if (IsPlug()) pluginParam = strcpyrealloc(pluginParam, parambuf);
 
         free(parambuf);
         free(sourcebuf);
@@ -836,7 +885,7 @@ bool cChannel::Save(FILE *f)
   return fprintf(f, "%s", *ToText()) > 0;
 }
 
-// -- cChannelSorter ---------------------------------------------------------
+// --- cChannelSorter --------------------------------------------------------
 
 class cChannelSorter : public cListObject {
 public:
@@ -852,7 +901,7 @@ public:
     }
   };
 
-// -- cChannels --------------------------------------------------------------
+// --- cChannels -------------------------------------------------------------
 
 cChannels Channels;
 
@@ -933,9 +982,10 @@ int cChannels::GetPrevNormal(int Idx)
   return channel ? Idx : -1;
 }
 
-void cChannels::ReNumber( void )
+void cChannels::ReNumber(void)
 {
   channelsHashSid.Clear();
+  maxNumber = 0;
   int Number = 1;
   for (cChannel *channel = First(); channel; channel = Next(channel)) {
       if (channel->GroupSep()) {
