@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: menuitems.c 1.49 2007/06/08 15:16:38 kls Exp $
+ * $Id: menuitems.c 1.58 2008/02/10 16:03:30 kls Exp $
  */
 
 #include "menuitems.h"
@@ -18,7 +18,7 @@
 
 #define AUTO_ADVANCE_TIMEOUT  1500 // ms before auto advance when entering characters via numeric keys
 
-const char *FileNameChars = " abcdefghijklmnopqrstuvwxyz0123456789-.#~,/_@";
+const char *FileNameChars = trNOOP("FileNameChars$ abcdefghijklmnopqrstuvwxyz0123456789-.,#~\\^$[]|()*+?{}/:%@&");
 
 // --- cMenuEditItem ---------------------------------------------------------
 
@@ -34,9 +34,8 @@ cMenuEditItem::~cMenuEditItem()
 
 void cMenuEditItem::SetValue(const char *Value)
 {
-  char *buffer = NULL;
-  asprintf(&buffer, "%s:\t%s", name, Value);
-  SetText(buffer, false);
+  cString buffer = cString::sprintf("%s:\t%s", name, Value);
+  SetText(buffer);
   cStatus::MsgOsdCurrentItem(buffer);
 }
 
@@ -222,7 +221,8 @@ cMenuEditChrItem::~cMenuEditChrItem()
 void cMenuEditChrItem::Set(void)
 {
   char buf[2];
-  snprintf(buf, sizeof(buf), "%c", *value);
+  buf[0] = *value;
+  buf[1] = '\0';
   SetValue(buf);
 }
 
@@ -255,7 +255,7 @@ cMenuEditStrItem::cMenuEditStrItem(const char *Name, char *Value, int Length, co
 {
   value = Value;
   length = Length;
-  allowed = Allowed;
+  allowed = Allowed ? Allowed : tr(FileNameChars);
   pos = -1;
   offset = 0;
   insert = uppercase = false;
@@ -284,7 +284,7 @@ void cMenuEditStrItem::EnterEditMode(void)
      int l = strlen(allowed) + 1;
      allowedUtf8 = new uint[l];
      Utf8ToArray(allowed, allowedUtf8, l);
-     const char *charMap = tr(" 0\t-.#~,/_@1\tabc2\tdef3\tghi4\tjkl5\tmno6\tpqrs7\ttuv8\twxyz9");
+     const char *charMap = tr("CharMap$ 0\t-.,1#~\\^$[]|()*+?{}/:%@&\tabc2\tdef3\tghi4\tjkl5\tmno6\tpqrs7\ttuv8\twxyz9");
      l = strlen(charMap) + 1;
      charMapUtf8 = new uint[l];
      Utf8ToArray(charMap, charMapUtf8, l);
@@ -301,11 +301,11 @@ void cMenuEditStrItem::LeaveEditMode(bool SaveValue)
         stripspace(value);
         }
      lengthUtf8 = 0;
-     delete valueUtf8;
+     delete[] valueUtf8;
      valueUtf8 = NULL;
-     delete allowedUtf8;
+     delete[] allowedUtf8;
      allowedUtf8 = NULL;
-     delete charMapUtf8;
+     delete[] charMapUtf8;
      charMapUtf8 = NULL;
      pos = -1;
      offset = 0;
@@ -316,7 +316,7 @@ void cMenuEditStrItem::LeaveEditMode(bool SaveValue)
 void cMenuEditStrItem::SetHelpKeys(void)
 {
   if (InEditMode())
-     cSkinDisplay::Current()->SetButtons(tr("Button$ABC/abc"), tr(insert ? "Button$Overwrite" : "Button$Insert"), tr("Button$Delete"));
+     cSkinDisplay::Current()->SetButtons(tr("Button$ABC/abc"), insert ? tr("Button$Overwrite") : tr("Button$Insert"), tr("Button$Delete"));
   else
      cSkinDisplay::Current()->SetButtons(NULL);
 }
@@ -490,9 +490,7 @@ eOSState cMenuEditStrItem::ProcessKey(eKeys Key)
                  break;
     case kBlue|k_Repeat:
     case kBlue:  // consume the key only if in edit-mode
-                 if (InEditMode())
-                    ;
-                 else
+                 if (!InEditMode())
                     return osUnknown;
                  break;
     case kLeft|k_Repeat:
@@ -500,9 +498,9 @@ eOSState cMenuEditStrItem::ProcessKey(eKeys Key)
                     if (!insert || newchar)
                        pos--;
                     newchar = true;
+                    if (!insert && Utf8is(alpha, valueUtf8[pos]))
+                       uppercase = Utf8is(upper, valueUtf8[pos]);
                     }
-                 if (!insert && Utf8is(alpha, valueUtf8[pos]))
-                    uppercase = Utf8is(upper, valueUtf8[pos]);
                  break;
     case kRight|k_Repeat:
     case kRight: if (InEditMode())
@@ -637,10 +635,10 @@ void cMenuEditRecPathItem::SetBase(const char* Path)
 {
   if (!Path)
       base[0] = 0;
-  strn0cpy(base, Path, sizeof(base));
+  Utf8Strn0Cpy(base, Path, sizeof(base));
   char* p = strrchr(base, '~');
   if (p)
-     p[0] = 0; 
+     p[0] = 0;
   else
      base[0] = 0;
 }
@@ -652,9 +650,10 @@ void cMenuEditRecPathItem::FindNextLevel()
   for (cRecording *recording = Recordings.First(); recording; recording = Recordings.Next(recording))
   {
      char* p;
-     strn0cpy(item, recording->Name(), sizeof(item));
+     Utf8Strn0Cpy(item, recording->Name(), sizeof(item));
      stripspace(value);
-     if (!strlen(value))
+     lengthUtf8 = Utf8ToArray(value, valueUtf8, length);
+     if (!lengthUtf8)
         p = strchr(item, '~');
      else {
         if (strstr(item, value) != item)
@@ -666,8 +665,9 @@ void cMenuEditRecPathItem::FindNextLevel()
      if (!p)
         continue;
      p[0] = 0;
-     strn0cpy(base, value, length);
-     strn0cpy(value, item, length);
+     Utf8Strn0Cpy(base, value, length);
+     Utf8Strn0Cpy(value, item, length);
+     lengthUtf8 = Utf8ToArray(value, valueUtf8, length);
      return;
      }
 }
@@ -680,26 +680,28 @@ void cMenuEditRecPathItem::Find(bool Next)
   for (cRecording *recording = Recordings.First(); recording; recording = Recordings.Next(recording))
   {
      const char* recName = recording->Name();
-     if (strlen(base) && strstr(recName, base) != recName)
+     if (Utf8StrLen(base) && strstr(recName, base) != recName)
         continue;
      if (strlen(base) && recName[strlen(base)] != '~')
         continue;
-     strn0cpy(item, recName, sizeof(item));
+     Utf8Strn0Cpy(item, recName, sizeof(item));
      char* p = strchr(item + strlen(base) + 1, '~');
      if (!p)
         continue;
      p[0] = 0;
      if (!Next && (strcmp(item, value) == 0)) {
         if (strlen(lastItem))
-           strn0cpy(value, lastItem, length);
+           Utf8Strn0Cpy(value, lastItem, length);
+        lengthUtf8 = Utf8ToArray(value, valueUtf8, length);
         return;
         }
      if (strcmp(lastItem, item) != 0) {
-        if(Next && strlen(lastItem) && strcmp(lastItem, value) == 0) {
-           strn0cpy(value, item, length);
+        if(Next && Utf8StrLen(lastItem) && strcmp(lastItem, value) == 0) {
+           Utf8Strn0Cpy(value, item, length);
+           lengthUtf8 = Utf8ToArray(value, valueUtf8, length);
            return;
            }
-        strn0cpy(lastItem, item, sizeof(lastItem));
+        Utf8Strn0Cpy(lastItem, item, sizeof(lastItem));
         }
      }
 }
@@ -716,21 +718,28 @@ eOSState cMenuEditRecPathItem::ProcessKey(eKeys Key)
     case kRed:    // one level up
                   if (!InEditMode())
                      return cMenuEditItem::ProcessKey(Key);
-                  strn0cpy(value, base, length);
+                  Utf8Strn0Cpy(value, base, lengthUtf8);
+                  lengthUtf8 = Utf8ToArray(value, valueUtf8, length);
                   SetBase(base);
-                  pos = strlen(base);
+                  pos = Utf8StrLen(base);
                   if (pos)
                      pos++;
-                  if (!strlen(value))
-                     strn0cpy(value, " ", length);
+                  if (!lengthUtf8) {
+                     Utf8Strn0Cpy(value, " ", length);
+                     lengthUtf8 = Utf8ToArray(value, valueUtf8, length);
+                     }
                   break;
     case kRight:
     case kGreen:  // one level down
                   if (InEditMode())
                      FindNextLevel();
-                  if (!strlen(value))
-                     strn0cpy(value, " ", length);
-                  pos = strlen(base);
+                  else
+                     EnterEditMode();
+                  if (!lengthUtf8) {
+                     Utf8Strn0Cpy(value, " ", length);
+                     lengthUtf8 = Utf8ToArray(value, valueUtf8, length);
+                     }
+                  pos = Utf8StrLen(base);
                   if (pos)
                      pos++;
                   SetHelpKeys();
@@ -742,7 +751,7 @@ eOSState cMenuEditRecPathItem::ProcessKey(eKeys Key)
                   if (!InEditMode())
                      return cMenuEditItem::ProcessKey(Key);
                   Find(false);
-                  pos = strlen(base);
+                  pos = Utf8StrLen(base);
                   if (pos)
                      pos++;
                   break;
@@ -753,7 +762,7 @@ eOSState cMenuEditRecPathItem::ProcessKey(eKeys Key)
                   if (!InEditMode())
                      return cMenuEditItem::ProcessKey(Key);
                   Find(true);
-                  pos = strlen(base);
+                  pos = Utf8StrLen(base);
                   if (pos)
                      pos++;
                   break;
@@ -761,8 +770,9 @@ eOSState cMenuEditRecPathItem::ProcessKey(eKeys Key)
                   if (!InEditMode())
                      return cMenuEditItem::ProcessKey(Key);
                   stripspace(value);
+                  lengthUtf8 = Utf8ToArray(value, valueUtf8, length);
                   cSkinDisplay::Current()->SetButtons(NULL);
-                  pos = -1;
+                  LeaveEditMode(Key == kOk);
                   break;
     default:
                   return cMenuEditItem::ProcessKey(Key);
